﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Storage;
using System.IO;
using SQLite;

namespace WP_DAO.BaseDades
{
    /// <summary>
    /// Classe encarregada d'interactuar i gestionar la base de dades local.
    /// </summary>
    internal class Main
    {
        public static string DB_PATH = Path.Combine(Path.Combine(ApplicationData.Current.LocalFolder.Path, "bd.sqlite"));

        private SQLiteConnection sqlconn;

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public Main()
        {
            sqlconn = new SQLiteConnection(DB_PATH);
            this.CreaBaseDades();
        }
        /// <summary>
        /// Funció encarregada de crear les taules a la base de dades.
        /// </summary>
        internal void CreaBaseDades()
        {
            sqlconn.CreateTable(new WP8_Configuration().GetType());
            sqlconn.CreateTable(new WP8_Usuari().GetType());
            sqlconn.CreateTable(new WP8_Operari().GetType());
            sqlconn.CreateTable(new WP8_Vehicle().GetType());
            sqlconn.CreateTable(new WP8_Client().GetType());
            sqlconn.CreateTable(new WP8_Imatge().GetType());
            sqlconn.CreateTable(new WP8_LiniaPressupost().GetType());
            sqlconn.CreateTable(typeof(WP8_Pressupost));
            sqlconn.CreateTable(typeof(WP8_VehiclesPressupost));
            sqlconn.CreateTable(typeof(WP8_OperarisPressupost));
        }
        /// <summary>
        /// Funció encarregada d'inserir un objecte a la base de dades.
        /// </summary>
        internal void InsertObject(object newObject)
        {
            sqlconn.InsertOrReplace(newObject);
        }
        /// <summary>
        /// Funció encarregada d'esborrar completament un pressupost de la base de dades.
        /// </summary>
        internal void DeletePressupost(int pressupostId)
        {
            sqlconn.Delete<WP8_Pressupost>(pressupostId);

            List<WP8_OperarisPressupost> operaris = sqlconn.Table<WP8_OperarisPressupost>().Where(c => c.PressupostId == pressupostId).ToList();
            List<int> operarisPressupostId = (from i in operaris select i.Id).ToList();
            foreach (int operariPressupostId in operarisPressupostId)
            {
                sqlconn.Delete<WP8_OperarisPressupost>(operariPressupostId);
            }

            List<WP8_VehiclesPressupost> vehicles = sqlconn.Table<WP8_VehiclesPressupost>().Where(c => c.PressupostId == pressupostId).ToList();
            List<int> vehiclesPressupostId = (from i in vehicles select i.Id).ToList();
            foreach (int vehiclePressupostId in vehiclesPressupostId)
            {
                sqlconn.Delete<WP8_VehiclesPressupost>(vehiclePressupostId);
            }

            IEnumerable<WP8_LiniaPressupost> linies = GetLinies(pressupostId);
            List<int> liniesId = (from i in linies select i.Id).ToList();
            foreach (int liniaId in liniesId)
            {
                sqlconn.Delete<WP8_LiniaPressupost>(liniaId);

                IEnumerable<WP8_Imatge> imatges = GetImatges(liniaId);
                List<int> imatgesId = (from i in imatges select i.Id).ToList();
                foreach (int imatgeId in imatgesId)
                {
                    sqlconn.Delete<WP8_Imatge>(imatgeId);
                }
            }

        }

        /// <summary>
        /// Funció encarregada de retornar un usuari en funció del seu identificador únic.
        /// </summary>
        internal WP8_Usuari GetUser(int id)
        {
            return sqlconn.Get<WP8_Usuari>(id);
        }
        /// <summary>
        /// Funció encarregada de retornar un client en funció del seu identificador únic.
        /// </summary>
        internal WP8_Client GetClient(int id)
        {
            return sqlconn.Get<WP8_Client>(id);
        }
        /// <summary>
        /// Funció encarregada de retornar els vehicles associats a un pressupost.
        /// </summary>
        internal IEnumerable<WP8_Vehicle> GetVehicles(int pressupostId)
        {
            List<WP8_VehiclesPressupost> vehicles = sqlconn.Table<WP8_VehiclesPressupost>().Where(c=> c.PressupostId == pressupostId).ToList();
            List<int> vehiclesId = (from i in vehicles select i.VehicleId).ToList();
            return sqlconn.Table<WP8_Vehicle>().Where(c => vehiclesId.Contains(c.Id));
        }
        /// <summary>
        /// Funció encarregada de retornar els operaris associats a un pressupost.
        /// </summary>
        internal IEnumerable<WP8_Operari> GetOperaris(int pressupostId)
        {
            List<WP8_OperarisPressupost> operaris = sqlconn.Table<WP8_OperarisPressupost>().Where(c => c.PressupostId == pressupostId).ToList();
            List<int> operarisId = (from i in operaris select i.OperariId).ToList();
            return sqlconn.Table<WP8_Operari>().Where(c => operarisId.Contains(c.Id));
        }
        /// <summary>
        /// Funció encarregada de retornar les línies corresponents a un pressupost.
        /// </summary>
        internal IEnumerable<WP8_LiniaPressupost> GetLinies(int pressupostId)
        {
            return sqlconn.Table<WP8_LiniaPressupost>().Where(c => c.PressupostId == pressupostId);
        }
        /// <summary>
        /// Funció encarregada de retornar les imatges associades a una línia de pressupost.
        /// </summary>
        internal IEnumerable<WP8_Imatge> GetImatges(int liniaPressupostId)
        {
            return sqlconn.Table<WP8_Imatge>().Where(c => c.LiniaPressupost == liniaPressupostId);
        }

        /// <summary>
        /// Funció encarregada de retornar la configuració emmagatzemada a la base de dades.
        /// </summary>
        internal WP8_Configuration CarregaConfiguracio()
        {
            return sqlconn.Table<WP8_Configuration>().FirstOrDefault();
        }
        /// <summary>
        /// Funció encarregada d'actualitzar a la base de dades la configuració passada.
        /// </summary>
        internal void GuardaConfiguracio(WP8_Configuration config)
        {
            sqlconn.InsertOrReplace(config);
        }

        /// <summary>
        /// Funció encarregada de validar l'usuari contra la base de dades local en funció del seu usuari i contrassenya.
        /// Si es correcte, retorna l'usuari. En cas contrari NULL.
        /// </summary>
        internal WP8_Usuari ValidaUsuari(string _nomUsuari, string _password)
        {
            return sqlconn.Table<WP8_Usuari>().Where(c => c.nom == _nomUsuari && c.password == _password).FirstOrDefault();
        }
        /// <summary>
        /// Funció encarregada de retornar els pressupostos pendents d'aprovació i assignats a l'usuari passat
        /// com a paràmetre.
        /// </summary>
        internal IEnumerable<WP8_Pressupost> GetPressupostosUsuari(int usuariId)
        {
            return sqlconn.Table<WP8_Pressupost>().Where(c => c.UsuariId == usuariId && c.acceptat == false).ToList();
        }
        /// <summary>
        /// Funció encarregada de retornar els pressupostos aprovats i assignats a l'usuari passat
        /// com a paràmetre.
        /// </summary>
        internal IEnumerable<WP8_Pressupost> GetTrasllatsUsuari(int usuariId)
        {
            return sqlconn.Table<WP8_Pressupost>().Where(c => c.UsuariId == usuariId && c.acceptat == true).ToList();
        }
        /// <summary>
        /// Funció encarregada de retornar els pressupostos que s'han modificat al terminal.
        /// </summary>
        internal IEnumerable<WP8_Pressupost> GetPressupostosActualitzats()
        {
            return sqlconn.Table<WP8_Pressupost>().Where(c => c.Actualitzat == true).ToList();
        }
    }
}
